%  Seriova a paralelni aplikace Cramerova pravidla 
%  Predpokladam, ze Matlab pool je jiz aktivni.
%  Budeme vyuzivat
% - loop variable (i)
% - sliced variable A,B,x
% -  temporary variable Ai,detAi

function CramerPool()
format long;
clear;clc;
N0 = 5;
Niter = 10;
Nmax = 250;

times = zeros(3,length(N0:Niter:Nmax));
iter = 1;
for N=N0:Niter:Nmax
    
    min = -3;
    max= 3;
    A = randi([min,max],N);
    B = randi([min,max],1,N)';
    xcramer = zeros(N,1);
    
    fprintf('\n-----------------------------------------------\n')
    fprintf('\nPocet radku puvodni matice %f\n',N)
    fprintf('\nHodnost matice %f\n',rank(A))
    fprintf('\nDeterminant matice %E\n',det(A))
    
    detA = det(A);
    Ainv = inv(A);
    xteor = Ainv*B;
    
    if(detA ~= 0)
        tic;
        [xcramer] = CramerSerial(A,B,N,N,xcramer);
        ends = toc;
        
        tic;
        [xcramer] = CramerParallel(A,B,N,N,xcramer);
        endp = toc;
        
        fprintf('\nSeriovy vypocet pro N = %d trval %f\n',N,ends)
        fprintf('\nParalelni vypocet pro N = %d trval %f\n',N,endp)
        fprintf('\n-----------------------------------------------\n')        
        
        times(1,iter) = N;
        times(2,iter) = ends;
        times(3,iter) = endp; 
        iter = iter+1;
    else
        % TODO podminku co se stabne pokud bude detA = 0
        
    end
    
    
end

plot(times(1,:),times(2,:),'-*b'); hold on
plot(times(1,:),times(3,:),'-*r'); hold on
xlabel('Velikost pole N'); ylabel('Cas vypoctu')
legend('Seriove','paralelne')
end

function [x] = CramerSerial(A,bvec,m,n,x)
% determinant matice musi byt nenulovy
% det(A) /= 0 - pak je jedno reseni, matice je regularni
% nahradime sloupec i vektorem b
% x_i = det(A_i)/det(A)
detA = det(A);
%     aplikuji Cramerovo pravidlo

Ai = A; % nova matice, kterou budu menit v cyklu

for i=1:n % loop pres vsechny sloupce
    for j=1:m % vymen vsechny prvky ve sloupci i
        Ai(j,i) = bvec(j);
    end
    %    spocitej determinant Ai
    
    detAi = det(Ai);
    x(i) = detAi/detA;
    %  kopiruj zpet puvodni matici
    Ai = A;
end

end


function [x] = CramerParallel(A,bvec,m,n,x)

% determinant matice musi byt nenulovy
% det(A) /= 0 - pak je jedno reseni, matice je regularni
% nahradime sloupec i vektorem b
% x_i = det(A_i)/det(A)

detA = det(A);
%     aplikuji Cramerovo pravidlo
parfor i=1:n % loop pres vsechny sloupce
    
    Ai = A; % nova matice, kterou budu menit v cyklu - v parfor musi byt pristupna pro kazdy worker
    
    for j=1:m % vymen vsechny prvky ve sloupci i
        Ai(j,i) = bvec(j);
    end
    %    spocitej determinant Ai
    
    detAi = det(Ai);
    x(i) = detAi/detA;
    %  kopiruj zpet puvodni matici
end

end
